@@ -59,170 +59,9 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         """
         # Many variables, branches, due to the many cases, pylint: disable=too-many-branches,too-many-statements
 
-        top_level_package_name = full_name.getTopLevelPackageName()
-
-        if top_level_package_name == "PyQt5":
-            if python_version < 0x300:
-                yield "atexit"
-
-            # These are alternatives depending on PyQt5 version
-            if full_name == "PyQt5.QtCore":
-                yield "sip"
-                yield "PyQt5.sip"
-
-            _, child = full_name.splitPackageName()
-
-            def getChildNamed(*child_names):
-                for child_name in child_names:
-                    return top_level_package_name.getChildNamed(child_name)
-
-            if child in (
-                "QtGui",
-                "QtAssistant",
-                "QtDBus",
-                "QtDeclarative",
-                "QtSql",
-                "QtDesigner",
-                "QtHelp",
-                "QtNetwork",
-                "QtScript",
-                "QtQml",
-                "QtGui",
-                "QtScriptTools",
-                "QtSvg",
-                "QtTest",
-                "QtWebKit",
-                "QtOpenGL",
-                "QtXml",
-                "QtXmlPatterns",
-                "QtPrintSupport",
-                "QtNfc",
-                "QtWebKitWidgets",
-                "QtBluetooth",
-                "QtMultimediaWidgets",
-                "QtQuick",
-                "QtWebChannel",
-                "QtWebSockets",
-                "QtX11Extras",
-                "_QOpenGLFunctions_2_0",
-                "_QOpenGLFunctions_2_1",
-                "_QOpenGLFunctions_4_1_Core",
-            ):
-                yield getChildNamed("QtCore")
-
-            if child in (
-                "QtDeclarative",
-                "QtWebKit",
-                "QtXmlPatterns",
-                "QtQml",
-                "QtPrintSupport",
-                "QtWebKitWidgets",
-                "QtMultimedia",
-                "QtMultimediaWidgets",
-                "QtQuick",
-                "QtQuickWidgets",
-                "QtWebSockets",
-                "QtWebEngineWidgets",
-            ):
-                yield getChildNamed("QtNetwork")
-
-            if child == "QtWebEngineWidgets":
-                yield getChildNamed("QtWebEngineCore")
-                yield getChildNamed("QtWebChannel")
-                yield getChildNamed("QtPrintSupport")
-            elif child == "QtScriptTools":
-                yield getChildNamed("QtScript")
-            elif child in (
-                "QtWidgets",
-                "QtDeclarative",
-                "QtDesigner",
-                "QtHelp",
-                "QtScriptTools",
-                "QtSvg",
-                "QtTest",
-                "QtWebKit",
-                "QtPrintSupport",
-                "QtWebKitWidgets",
-                "QtMultimedia",
-                "QtMultimediaWidgets",
-                "QtOpenGL",
-                "QtQuick",
-                "QtQuickWidgets",
-                "QtSql",
-                "_QOpenGLFunctions_2_0",
-                "_QOpenGLFunctions_2_1",
-                "_QOpenGLFunctions_4_1_Core",
-            ):
-                yield getChildNamed("QtGui")
-
-            if full_name in (
-                "PyQt5.QtDesigner",
-                "PyQt5.QtHelp",
-                "PyQt5.QtTest",
-                "PyQt5.QtPrintSupport",
-                "PyQt5.QtSvg",
-                "PyQt5.QtOpenGL",
-                "PyQt5.QtWebKitWidgets",
-                "PyQt5.QtMultimediaWidgets",
-                "PyQt5.QtQuickWidgets",
-                "PyQt5.QtSql",
-            ):
-                yield "PyQt5.QtWidgets"
-
-            if full_name in ("PyQt5.QtPrintSupport",):
-                yield "PyQt5.QtSvg"
-
-            if full_name in ("PyQt5.QtWebKitWidgets",):
-                yield "PyQt5.QtWebKit"
-                yield "PyQt5.QtPrintSupport"
-
-            if full_name in ("PyQt5.QtMultimediaWidgets",):
-                yield "PyQt5.QtMultimedia"
-
-            if full_name in ("PyQt5.QtQuick", "PyQt5.QtQuickWidgets"):
-                yield "PyQt5.QtQml"
-
-            if full_name in ("PyQt5.QtQuickWidgets", "PyQt5.QtQml"):
-                yield "PyQt5.QtQuick"
-
-            if full_name == "PyQt5.Qt":
-                yield "PyQt5.QtCore"
-                yield "PyQt5.QtDBus"
-                yield "PyQt5.QtGui"
-                yield "PyQt5.QtNetwork"
-                yield "PyQt5.QtNetworkAuth"
-                yield "PyQt5.QtSensors"
-                yield "PyQt5.QtSerialPort"
-                yield "PyQt5.QtMultimedia"
-                yield "PyQt5.QtQml"
-                yield "PyQt5.QtWidgets"
-
-        elif full_name == "sip" and python_version < 0x300:
+        if full_name == "sip" and python_version < 0x300:
             yield "enum"
 
-        elif full_name == "PySide.QtDeclarative":
-            yield "PySide.QtGui"
-        elif full_name == "PySide.QtHelp":
-            yield "PySide.QtGui"
-        elif full_name == "PySide.QtOpenGL":
-            yield "PySide.QtGui"
-        elif full_name == "PySide.QtScriptTools":
-            yield "PySide.QtScript"
-            yield "PySide.QtGui"
-        elif full_name == "PySide.QtSql":
-            yield "PySide.QtGui"
-        elif full_name == "PySide.QtSvg":
-            yield "PySide.QtGui"
-        elif full_name == "PySide.QtTest":
-            yield "PySide.QtGui"
-        elif full_name == "PySide.QtUiTools":
-            yield "PySide.QtGui"
-            yield "PySide.QtXml"
-        elif full_name == "PySide.QtWebKit":
-            yield "PySide.QtGui"
-        elif full_name == "PySide.phonon":
-            yield "PySide.QtGui"
-
         elif full_name == "lxml":
             yield "lxml.builder"
             yield "lxml.etree"
@@ -1092,6 +931,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "pandas._libs.tslibs.nattype"
             yield "pandas._libs.tslibs.base"
         elif full_name == "pandas.core.window":
+            yield "pandas._libs.window"
             yield "pandas._libs.skiplist"
         elif full_name == "pandas._libs.testing":
             yield "cmath"
@@ -1186,6 +1026,9 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
             yield "py._path.local"
         elif full_name == "pyreadstat._readstat_parser":
             yield "pandas"
+        elif full_name == "pyreadstat.pyreadstat":
+            yield "pyreadstat._readstat_writer"
+            yield "pyreadstat.worker"
         elif full_name == "cytoolz.itertoolz":
             yield "cytoolz.utils"
         elif full_name == "cytoolz.functoolz":
@@ -1495,6 +1338,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         "wx._core",  # Too large generated code
         "pyVmomi.ServerObjects",  # Too large generated code
         "pyglet.gl",  # Too large generated code
+        "telethon.tl.types",  # Not performance relevant and slow C compile
     )
 
     def decideCompilation(self, module_name, source_ref):
